# ~*~ coding: utf-8

from Products.Five import BrowserView
from zope.publisher.interfaces.browser import IBrowserView
from zope.interface import implements
from zope.component import getUtility
from Products.Korpus.schemas import get_field, get_schema, all
from Products.Korpus.utils import translate
texttypeclass = get_field('texttype')

from Products.Korpus.interfaces import IWWW, ICorpusManager
from App import config as zconfig
kc = zconfig.getConfiguration().product_config.get('korpus', {})
inprod = kc.get('in_production', False)

class View(BrowserView):

    implements(IBrowserView)

    def __init__(self, context, request):
        self.request = request
        self.context = context
        self.wwwpage = ""

    def test(self, cond, tru, fal=''):
        if cond: return tru
        return fal

    def getTextTypesHTML(self):
        return texttypeclass.select(self.request, self.context.getLanguage(self.request))

    def getInformantView(self):
        sc = get_schema(self.context.getMetaSchema())
        return sc.view(self.context, self.context.getLanguage(self.request))

    def getInformantEdit(self):
        sc = get_schema(self.context.getMetaSchema())
        return sc.edit(self.context, self.context.getLanguage(self.request))

    def setPage(self, pname):
        self.wwwpage = pname

    def getPage(self):
        return self.wwwpage

    def getPageLinks(self):
        lang = self.context.getLanguage(self.request)
        w = getUtility(IWWW)
        return w.getPagesList(lang)

    def getPortalLanguages(self):
        return getattr(self.context, 'portallangs', [])

    def getJavascriptLink(self):
        suf = ""
        if not inprod or inprod == "False": suf = "-dev"
        return """
    <script type="text/javascript" src="http://evkk.tlu.ee/static/mootools-core%s.js"></script>
    <script type="text/javascript" src="http://evkk.tlu.ee/static/mootools-more%s.js"></script>
    <script type="text/javascript" src="++resource++js/korpus.js"></script>
    """ % (suf, suf)

    def useTagging(self):
        return kc.get('use_tagging', False) != "False"

class StatisticsView(View):
    """ statistics view """

    def renderStatisticsTable(self):
        seen = []
        res = u""
        stats = self.context.compute_stats(self.request)
        base = self.request.get('base', None)
        lang = self.context.getLanguage(self.request)

        fields = []
        for s in all():
            for f in s.fields:
                if f.getName() in seen: continue
                seen.append(f.getName())
                fields.append([len(f.choices), f])
        fields.sort(reverse=False)
        x = fields[-1]
        fields[-1] = fields[-3]
        fields[-3] = x
        for num_of_choices, f in fields:
            tmp = u'<table class="statsField">'
            tmp += '<tr>'
            tmp += '<th class="statsHeader">'+f.getTitleI18N(lang)+'</th>'
            tmp += '<th class="statsHeader">%</th>'
            tmp += '<th class="statsHeader">'+translate(None, 'words', lang)+'</th>'
            tmp += '</tr>'
            for k, v in f.getItems():
                cls = ''
                if base is not None:
                    if base == f.getName()+'_'+k:
                        cls = ' class="viewPoint"'
                tmp += '<tr'+cls+'>'
                tmp += '<td>'
                if not cls: tmp += '<a href="?base='+f.getName()+'_'+k+'">'
                im = f.getChoiceI18N(k, lang)
                tmp += im
                if not cls: tmp += '</a>'
                tmp += '</td>'
                perc = stats.get(f.getName()+'_'+k+'_perc')
                if perc is None: perc = 0
                word = stats.get(f.getName()+'_'+k+'_word')
                if word is None: word = 0
                tmp += '<td>'+str(perc)+'</td>'
                tmp += '<td>'+str(word)+'</td>'
                tmp += '</tr>'
            tmp += '</table>'
            res += tmp
        return res

class SearchView(View):
    """ search view """

    def renderSearchForm(self):
        """ render search form """
        res = u""
        lang = self.context.getLanguage(self.request)
        fc = 0
        seen = []
        for s in all():
            res += '<tr>\n'
            for f in s.fields:
                if f.getName() in seen: continue
                seen.append(f.getName())

                fc = fc + 1
                if fc == 3:
                    res += '</tr>\n<tr>\n'
                    fc = 1

                res += '<td style="text-align: right;">\n'
                res += f.getTitleI18N(lang)
                res += '</td>\n'
                res += '<td style="text-align: left;">\n'
                res += f.select(trLang = lang)
                res += '</td>\n'
            res += '</tr>\n'
        return res

class DocumentsView(View):

    def listDocus(self):
        d = ['document_1', 'document_2']
        return '\n'.join(d)

    def getSchemas(self):
        return all()

    def getCorpuses(self):
        cm = getUtility(ICorpusManager)
        return cm.getCorpuses()
