import re

class PipeIter(object):

    def __init__(self, obj):
        self.par = obj
        self._itercount = -1

    def next(self):
        if not len(self.par._ds) or self._itercount == len(self.par._ds) - 1:
            raise StopIteration
        self._itercount += 1
        return self.par._ds[self._itercount]

class Pipe(object):

    def __init__(self):
        self._ds = []
        self._rulematches = []

    def __iter__(self):
        return PipeIter(self)

    def __str__(self):
        return ' '.join([ w.getWord() for w in self ])

    def append(self, o):
        self._ds.append(o)

    def match(self, rule):
        r = False
        doContinue = False
        start = -1
        for rp in rule:
            for i, w in enumerate(self):
                if w.isIgnored(): continue
                if i < start: continue
                #print start, rp, rp not in w.getInfo(), doContinue, w.hasMarking(rp)
                if not w.hasMarking(rp):
                    doContinue = True
                else:
                    start = i + 1
                    break
        if not doContinue:
            r = True
        return r

    def addRuleMatch(self, rid):
        self._rulematches.append(rid)

    def getPossibleRule(self, searchables, debug=False):
        """ searchables - a list of allowed marks """
        rule = []
        for i, w in enumerate(self):
            if debug: print w.getInfo()
            if w.isIgnored():
                continue
            has_mark = False
            for s in searchables:
                if w.hasMarking(s):
                    if len(rule) == 1 and rule[0] == s:
                        pass
                    else:
                        rule.append(s)
                    has_mark = True
                    break
            if not has_mark and len(rule) > 0:
                print "show-stopper:", w.getWord(), w.getInfo()
                break
        return rule #re.sub(' +', ' ', rule)

class Word(object):

    def __init__(self, w):
        self.word = w
        self.ignored = False
        self.inf = []

    def __str__(self):
        if self.isIgnored(): ignf = '1'
        else: ignf = '0'
        return "<Word> %s %-20s %s" % (ignf, self.word, self.inf)

    def addInfo(self, inf):
        self.inf.append(inf)

    def getInfo(self):
        return self.inf

    def getWord(self):
        return self.word

    def isIgnored(self):
        return self.ignored

    def setIgnored(self, ign):
        assert isinstance(ign, bool)
        self.ignored = ign

    def hasMarking(self, mark):
        for i in self.inf:
            if mark in i:
                return True
        return False

class RulesIter(object):

    def __init__(self, par):
        self.par = par
        self.c = -1

    def next(self):
        self.c = self.c + 1
        if self.c == len(self.par._rids):
            raise StopIteration
        return [self.par._rids[self.c], self.par._rules[self.par._rids[self.c]]]

class RulesList(object):

    def __init__(self, _rlist):
        self._statistics = {}
        self._rids = []
        for x in _rlist:
            self._rids.append(x.keys()[0])
        self._rawrules = _rlist
        self._rules = {}
        for x in _rlist:
            self._rules[x.keys()[0]] = x[x.keys()[0]]

    def __iter__(self):
        return RulesIter(self)

    def printStats(self):
        print 
        s = self._statistics.items()
        s.sort(reverse=True, key=lambda x:x[1])
        total = 0
        for x in s:
            print str(x[1]).ljust(5), self._rules[x[0]]
            total += x[1]
        print "Total rule instances matched", total

    def get(self, rid):
        return self._rules[rid]

    def addStat(self, rid):
        if not self._statistics.has_key(rid):
            self._statistics[rid] = 0
        self._statistics[rid] = self._statistics[rid] + 1


if __name__ == '__main__':
    p = Pipe()
    print [ x for x in p ]
    p.append(1)
    p.append(2)
    l = getattr(p, 'append')
    print l
    print [ x for x in p ]
    for x in p:
        for y in p:
            print x, y

    print 1 in p
    w = Word('asd')
    w in ['1','2','3']

