
PG_CONN_ID = 'evkk_pg_conn'

def _setupSQL(self, connstring):
    if not hasattr(self, PG_CONN_ID):
        from Products.ZPsycopgDA.DA import manage_addZPsycopgConnection
        manage_addZPsycopgConnection(self, 
                PG_CONN_ID, 
                'PG connection', 
                connstring,
                None,   # zdatetime
                1,      # READ_COMMITED
                'UTF8')
    from Products.ZSQLMethods.SQL import SQL
    from Products.ZSQLMethods.SQL import manage_addZSQLMethod
    from OFS.Folder import manage_addFolder
    if not hasattr(self, 'sqls'):
        manage_addFolder(self, 'sqls', 'ZSQL Methods')
    sqls = getattr(self, 'sqls')

    # ================================================================= #
    # words table
    # ================================================================= #
    if not hasattr(sqls, 'wordInsert'):
        manage_addZSQLMethod(sqls, 'wordInsert', 
                'Insert a word in to words table', PG_CONN_ID,
                'word prim lemma category gram language raw',
                """
                INSERT INTO words VALUES (
                    NEXTVAL('widseq'),
                    <dtml-sqlvar word type="string">,
                    1,
                    <dtml-sqlvar prim type="string">,
                    <dtml-sqlvar lemma type="string">,
                    <dtml-sqlvar category type="string">,
                    <dtml-sqlvar gram type="string">,
                    <dtml-sqlvar language type="string">,
                    <dtml-sqlvar raw type="string">
                );
                """)
    if not hasattr(sqls, 'wordGetByWord'):
        manage_addZSQLMethod(sqls, 'wordGetByWord',
                "Get word by word", PG_CONN_ID,
                'word',
                """
                SELECT * FROM words WHERE word=<dtml-sqlvar word type="string">;
                """)
    if not hasattr(sqls, 'wordsByLemma'):
        manage_addZSQLMethod(sqls, 'wordsByLemma',
                "Get words by lemma", PG_CONN_ID,
                "lemma",
                """
                SELECT word, prim, lemma, category, gram FROM words
                WHERE lemma=<dtml-sqlvar lemma type="string">;
                """)
    if not hasattr(sqls, 'getNofWords'):
        manage_addZSQLMethod(sqls, 'getNofWords',
                "get a number of words with morf. info", PG_CONN_ID,
                "",
                """
                SELECT count("ID") FROM words;
                """)

    # ================================================================= #
    # undecided table
    # ================================================================= #

    # ================================================================= #
    # documents
    # ================================================================= #
    if not hasattr(sqls, 'docsInsert'):
        manage_addZSQLMethod(sqls, 'docsInsert',
                "register word presence in a document", PG_CONN_ID,
                "word docid language corpus",
                """
                INSERT INTO docrefs (word, documentid, "ID", language, corpus) VALUES (
                    <dtml-sqlvar word type="string">,
                    <dtml-sqlvar docid type="string">,
                    NEXTVAL('didseq'),
                    <dtml-sqlvar language type="string">,
                    <dtml-sqlvar corpus type="string">
                );
                """)
    if not hasattr(sqls, 'getWordsPerDoc'):
        manage_addZSQLMethod(sqls, 'getWordsPerDoc',
                "get all words from one document", PG_CONN_ID,
                "docid",
                """
                SELECT word FROM docrefs
                WHERE documentid=<dtml-sqlvar docid type="string">;
                """)
    if not hasattr(sqls, 'storeDocContent'):
        manage_addZSQLMethod(sqls, 'storeDocContent',
                "store document's ID and title", PG_CONN_ID,
                "docid html plaintext",
                """
                DELETE FROM texts WHERE documentid=<dtml-sqlvar docid type="string">
                <dtml-var sql_delimiter>
                INSERT INTO texts (documentid, html, plaintext) VALUES (
                    <dtml-sqlvar docid type="string">,
                    <dtml-sqlvar html type="string">,
                    <dtml-sqlvar plaintext type="string">
                );
                """)
    if not hasattr(sqls, 'storeDocument'):
        manage_addZSQLMethod(sqls, 'storeDocument',
                "store document's ID and title", PG_CONN_ID,
                "docid title language corpus",
                """
                DELETE FROM documents WHERE documentid=<dtml-sqlvar docid type="string">
                <dtml-var sql_delimiter>
                INSERT INTO documents (documentid, title, language, corpus) VALUES (
                    <dtml-sqlvar docid type="string">,
                    <dtml-sqlvar title type="string">,
                    <dtml-sqlvar language type="string">,
                    <dtml-sqlvar corpus type="string">
                );
                """)
    if not hasattr(sqls, 'deleteWordDocref'):
        manage_addZSQLMethod(sqls, 'deleteWordDocref',
                "delete doc ref from docrefs table", PG_CONN_ID,
                "docid word lim",
                """
                DELETE FROM docrefs
                WHERE documentid=<dtml-sqlvar docid type="string">
                AND word=<dtml-sqlvar word type="string">;
                """)
    if not hasattr(sqls, 'deleteDocref'):
        manage_addZSQLMethod(sqls, 'deleteDocref',
                "delete doc ref from docrefs table", PG_CONN_ID,
                "docid",
                """
                DELETE FROM docrefs WHERE documentid=<dtml-sqlvar docid type="string">;
                """)
    if not hasattr(sqls, 'selectFirstChars'):
        manage_addZSQLMethod(sqls, 'selectFirstChars', 
                "select first chars", PG_CONN_ID,
                "",                    
                """
                SELECT * FROM firstchars;
                """)
        sqls.selectFirstChars.manage_advanced(
                1000, # max_rows
                100, # max_cache
                300, # cache_time, 5 min
                '', '')

    # ================================================================= #
    # Get a list of words ordered by freq
    # Get a list of words ordered by alpha
    # ================================================================= #
    # SELECT word, count(documentid) AS count 
    # FROM docrefs WHERE word LIKE <dtml-sqlvar char type="string">
    # GROUP BY word 
    # ORDER BY <dtml-var order>;
    if not hasattr(sqls, 'getWords'):
        manage_addZSQLMethod(sqls, 'getWords', 
                "order by: word, count", PG_CONN_ID,
                "char order",
                """
                SELECT word, count('word'), lemma, prim FROM docrefs 
                LEFT JOIN words USING(word) 
                WHERE word LIKE <dtml-sqlvar char type="string"> 
                GROUP BY word, lemma, prim 
                ORDER BY <dtml-var order>, word;
                """)
    if not hasattr(sqls, 'getUndecided'):
        manage_addZSQLMethod(sqls, 'getUndecided',
                "get undecided words", PG_CONN_ID,
                "limit offset",
                """
                SELECT "ID", word, option, language FROM undecided 
                WHERE word IN (
                SELECT DISTINCT word FROM undecided WHERE processed=false GROUP BY word
                EXCEPT SELECT fromword FROM aliases
                LIMIT <dtml-var limit>
                OFFSET <dtml-var offset>);
                """)
    if not hasattr(sqls, 'getUndecidedFilter'):
        manage_addZSQLMethod(sqls, 'getUndecidedFilter',
                "get undecided words with limit option", PG_CONN_ID,
                "limit offset filter",
                """
                SELECT "ID", word, option, language FROM undecided 
                WHERE word IN (
                SELECT DISTINCT word FROM undecided WHERE processed=false AND
                word LIKE <dtml-var filter>
                GROUP BY word
                EXCEPT SELECT fromword FROM aliases
                LIMIT <dtml-var limit>
                OFFSET <dtml-var offset>);
                """)
    if not hasattr(sqls, 'getNofUndecided'):
        manage_addZSQLMethod(sqls, 'getNofUndecided',
                "get a number of undecided words", PG_CONN_ID,
                "",
                """
                SELECT count(idcount)
                FROM (SELECT count("ID") AS idcount FROM undecided GROUP BY word) AS grouped;
                """)
    if not hasattr(sqls, 'tickOption'):
        manage_addZSQLMethod(sqls, 'tickOption',
                "tick option in undecided table", PG_CONN_ID,
                "id",
                """
                UPDATE undecided SET tick=true WHERE "ID"=<dtml-var id>;
                """)
    if not hasattr(sqls, 'tickWithData'):
        manage_addZSQLMethod(sqls, 'tickWithData',
                "add an option in to undecided and tick it", PG_CONN_ID,
                "word option",
                """
                INSERT INTO undecided ("ID", word, option, language, tick) VALUES (
                    NEXTVAL('unidseq'), 
                    <dtml-sqlvar word type="string">, 
                    <dtml-sqlvar option type="string">, 
                    'ET', 
                    true
                );
                """)
    if not hasattr(sqls, 'addAlias'):
        manage_addZSQLMethod(sqls, 'addAlias',
                "link word with correct word", PG_CONN_ID,
                "fromword toword",
                """
                INSERT INTO aliases ("ID", fromword, toword) VALUES (
                    NEXTVAL('aliidseq'),
                    <dtml-sqlvar fromword type="string">,
                    <dtml-sqlvar toword type="string">
                );
                """)
    if not hasattr(sqls, 'getTotalWordCount'):
        manage_addZSQLMethod(sqls, 'getTotalWordCount',
                "total number of words", PG_CONN_ID,
                "",
                """
                SELECT count("ID") from docrefs;
                """)

    # to get documents and word count in them! rowcount show total number of documents where 
    # this word is in.
    if not hasattr(sqls, 'getDocWordStats'):
        manage_addZSQLMethod(sqls, 'getDocWordStats',
                "per word, doc statistics", PG_CONN_ID,
                "word",
                """
                SELECT documentid, count(documentid), title
                FROM docrefs 
                LEFT JOIN documents USING(documentid)
                WHERE word=<dtml-sqlvar word type="string">
                GROUP BY documentid, title
                ORDER BY count DESC;
                """)
    if not hasattr(sqls, 'getRefFrom'):
        manage_addZSQLMethod(sqls, 'getRefFrom',
                "get reference from word", PG_CONN_ID,
                "word",
                """
                SELECT toword FROM aliases WHERE fromword=<dtml-sqlvar word type="string">;
                """)
    if not hasattr(sqls, 'getRefTo'):
        manage_addZSQLMethod(sqls, 'getRefTo',
                "get reference words to word", PG_CONN_ID,
                "word",
                """
                SELECT fromword FROM aliases WHERE toword=<dtml-sqlvar word type="string">;
                """)
    # select documentid,count(documentid) from docrefs where word='akulaadija' group by documentid;

    if not hasattr(sqls, 'getWordCount'):
        manage_addZSQLMethod(sqls, 'getWordCount',
                "freq of one word", PG_CONN_ID,
                "word",
                """
                SELECT count("ID") FROM docrefs WHERE word=<dtml-sqlvar word type="string">;
                """)
    # ================================================================= #
    #
    # ================================================================= #
    #select documentid, count(documentid) from docrefs where word='lenin' group by documentid order by count desc;

    # ERRORS
    if not hasattr(sqls, 'newError'):
        manage_addZSQLMethod(sqls, 'newError',
                "add new error", PG_CONN_ID,
                "pointer documentid before content after code author",
                """
                INSERT INTO errors
                (pointer, documentid, before, content, after, code, author)
                VALUES
                (
                <dtml-sqlvar pointer type="string">,
                <dtml-sqlvar documentid type="string">,
                <dtml-sqlvar before type="string">,
                <dtml-sqlvar content type="string">,
                <dtml-sqlvar after type="string">,
                <dtml-sqlvar code type="string">,
                <dtml-sqlvar author type="string">
                )
                <dtml-var sql_delimiter>
                SELECT CURRVAL('errors_id_seq');
                """)
    if not hasattr(sqls, 'errDelete'):
        manage_addZSQLMethod(sqls, 'errDelete',
                "delete marked error", PG_CONN_ID,
                "errid",
                """
                UPDATE errors SET deleted='t' WHERE id=<dtml-sqlvar errid type="int">;
                """)

    if not hasattr(sqls, 'docDeleted'):
        manage_addZSQLMethod(sqls, 'docDeleted',
                "mark all doc related stuff deleted", PG_CONN_ID,
                "docid",
                """
                UPDATE docrefs SET deleted='t'
                WHERE documentid=<dtml-sqlvar docid type="string">
                <dtml-var sql_delimiter>
                UPDATE documents SET deleted='t'
                WHERE documentid=<dtml-sqlvar docid type="string">
                <dtml-var sql_delimiter>
                UPDATE texts SET deleted='t'
                WHERE documentid=<dtml-sqlvar docid type="string">
                <dtml-var sql_delimiter>
                UPDATE sentences SET deleted='t'
                WHERE documentid=<dtml-sqlvar docid type="string">;
                """)

    if not hasattr(sqls, 'docStatus'):
        manage_addZSQLMethod(sqls, 'docStatus',
                "set document status", PG_CONN_ID,
                "docid status",
                """
                UPDATE texts SET status=<dtml-sqlvar status type="int">
                WHERE documentid=<dtml-sqlvar docid type="string">;
                """)

    if not hasattr(sqls, 'morfoRaw'):
        manage_addZSQLMethod(sqls, 'morfoRaw',
                "get morfo analysis", PG_CONN_ID,
                "docid",
                """
                SELECT rawmorfo FROM analysis WHERE documentid=<dtml-sqlvar docid type="string">;
                """)

    if not hasattr(sqls, 'tagAdd'):
        manage_addZSQLMethod(sqls, 'tagAdd',
                "add a tag", PG_CONN_ID,
                "uname tag",
                """
                INSERT INTO tags (uname, label) VALUES (
                    <dtml-sqlvar uname type="string">,
                    <dtml-sqlvar tag type="string">
                )
                <dtml-var sql_delimiter>
                SELECT CURRVAL('tags_id_seq');
                """)
    if not hasattr(sqls, 'tagDelete'):
        manage_addZSQLMethod(sqls, 'tagDelete',
                "delete tag", PG_CONN_ID,
                "uname tagid",
                """
                DELETE FROM tags WHERE id=<dtml-sqlvar tagid type="int"> AND uname=<dtml-sqlvar uname type="string">;
                """)
    if not hasattr(sqls, 'tagGet'):
        manage_addZSQLMethod(sqls, 'tagGet',
                "get a tag by id", PG_CONN_ID,
                "tagid",
                """
                SELECT * FROM tags WHERE id=<dtml-sqlvar tagid type="int">;
                """)
    if not hasattr(sqls, 'tagDocument'):
        manage_addZSQLMethod(sqls, 'tagDocument',
                "tag document", PG_CONN_ID,
                "uname docid tagid",
                """
                INSERT INTO tagged (uname, documentid, tag) VALUES (
                    <dtml-sqlvar uname type="string">,
                    <dtml-sqlvar docid type="string">,
                    <dtml-sqlvar tagid type="int">
                )
                <dtml-var sql_delimiter>
                SELECT CURRVAL('tagged_id_seq');
                """)
    if not hasattr(sqls, 'tagDocumentRemove'):
        manage_addZSQLMethod(sqls, 'tagDocumentRemove',
                "untag a document", PG_CONN_ID,
                "uname tagid",
                """
                DELETE FROM tagged WHERE tag=<dtml-sqlvar tagid type="int">;
                """)
                #DELETE FROM tagged WHERE id=<dtml-sqlvar taggedid type="int"> AND uname=<dtml-sqlvar uname type="string">;
    if not hasattr(sqls, 'tagGetByDocument'):
        manage_addZSQLMethod(sqls, 'tagGetByDocument',
                "get tags by document id", PG_CONN_ID,
                "docid",
                """
                SELECT * FROM tagged WHERE documentid=<dtml-sqlvar docid type="string">;
                """)
    if not hasattr(sqls, 'tagsGet'):
        manage_addZSQLMethod(sqls, 'tagsGet',
                "get a list of tags", PG_CONN_ID,
                "uname",
                """
                SELECT *, uname=<dtml-sqlvar uname type="string"> AS mine FROM tags
                WHERE uname=<dtml-sqlvar uname type="string"> OR global='t' ORDER BY mine DESC;
                """)

    if not hasattr(sqls, 'addSentence'):
        manage_addZSQLMethod(sqls, 'addSentence',
                "add a sentence", PG_CONN_ID,
                "docid lang sentence",
                """
                INSERT INTO sentences (documentid, language, sentence) VALUES (
                    <dtml-sqlvar docid type="string">,
                    <dtml-sqlvar lang type="string">,
                    <dtml-sqlvar sentence type="string">
                )
                <dtml-var sql_delimiter>
                SELECT CURRVAL('sentences_id_seq');
                """)
    if not hasattr(sqls, 'deleteSentencesByDocument'):
        manage_addZSQLMethod(sqls, 'deleteSentencesByDocument',
                "delete sentences by document", PG_CONN_ID,
                "docid",
                """
                DELETE FROM sentences WHERE documentid=<dtml-sqlvar docid type="string">;
                """)

    return True


